@@ -3,7 +3,7 @@
 read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=NULL,columns=NULL,other.columns=NULL,annotation=NULL,wt.fun=NULL,verbose=TRUE,sep="\t",quote=NULL,DEBUG=FALSE,...)
 #	Extracts an RG list from a series of image analysis output files
 #	Gordon Smyth. 
-#	1 Nov 2002.  Last revised 22 October 2005.
+#	1 Nov 2002.  Last revised 17 Dec 2005.
 #	Use of colClasses added by Marcus Davy, 14 October 2005.
 {
 #	For checking colClasses setup
@@ -18,7 +18,7 @@ read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=N
 
 #	Begin check input arguments
 
-	if(missing(files)) {
+	if(is.null(files)) {
 		if(is.null(ext))
 			stop("Must specify input files")
 		else {
@@ -27,6 +27,14 @@ read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=N
 			files <- sub(extregex,"",files)
 		}
 	}
+	if(is.data.frame(files)) {
+		targets <- files
+		files <- files$FileName
+		if(is.null(files)) stop("targets frame doesn't contain FileName column")
+		if(is.null(names)) names <- targets$Label
+	} else {
+		targets <- NULL
+	}
 
 	source <- match.arg(source,c("generic","agilent","arrayvision","bluefuse","genepix","genepix.median","genepix.custom","imagene","quantarray","smd.old","smd","spot","spot.close.open"))
 	source2 <- strsplit(source,split=".",fixed=TRUE)[[1]][1]
@@ -142,7 +150,11 @@ read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=N
 	RG <- columns
 	for (a in cnames) RG[[a]] <- Y
 	if(!is.null(wt.fun)) RG$weights <- Y
-	RG$targets <- data.frame(FileName=I(files),row.names=names)
+	if(is.data.frame(targets)) {
+		RG$targets <- targets
+	} else {
+		RG$targets <- data.frame(FileName=I(files),row.names=names)
+	}
 
 #	Set annotation columns
 	if(!is.null(annotation)) {
@@ -231,47 +243,38 @@ read.maimages <- function(files=NULL,source="generic",path=NULL,ext=NULL,names=N
 }
 
 wtVariables <- function(x,fun)
-# Extracts variable names from weight functions such as wtarea, wtflags, 
-# wtIgnore.Filter and user defined functions
-# Gordon Smyth
-# 11 Apr 2004 (Renamed from variablesinfunction)
+#	Finds variable names in user-defined functions
+#	Gordon Smyth
+#	3 Nov 2004. Last modified 2 Jan 2006.
 {
-         x <- as.character(x)
-         a <- deparse(fun)
-         n <- length(x)
-         ind <- logical(n)
-         for (i in 1:n) {
-                 ind[i] <- as.logical(length(grep(x[i],a)))
-         }
-         x[ind]
+	x <- as.character(x)
+	a <- deparse(fun)
+	n <- length(x)
+	ind <- logical(n)
+	for (i in 1:n) {
+		ind[i] <- as.logical(length(grep(protectMetachar(x[i]),a)))
+	}
+	x[ind]
 }
 
 getColClasses <- function(cols, ...)
-# Construct a colClasses vector for read.table from a vector of possible columns 'cols' 
-# Uses wtVariables and ellipsis vectors and lists of character string variable names
-# to match against 'cols'
-# Marcus Davy 
-# 20 Apr 2004 Last revised 3 October 2005.
+#	Construct a colClasses vector for read.table from a vector of possible columns 'cols' 
+#	Uses wtVariables and ellipsis vectors and lists of character string variable names
+#	to match against 'cols'
+#	Marcus Davy 
+#	16 Nov 2004. Last revised 2 Jan 2006.
 {
-	if(is.list(cols)){
-        stop("arg 'cols' must be a vector")
-    }
-    cols     <- as.character(cols)
-  x        <- rep("NULL", length(cols))
-  names(x) <- cols
-  wanted   <- list(...)
-  for(i in 1:length(wanted)) {
-    if(is.null(wanted[[i]]))
-      next
-    if(is.function(wanted[[i]])) {
-      include <- wtVariables(cols, wanted[[i]])
-    } 
-    if(is.list(wanted[[i]])) 
-      wanted[[i]] <- unlist(wanted[[i]])
-    if(is.character(wanted[[i]])) 
-      include <- wanted[[i]]
-    ind <- match(include, cols, nomatch=0)
-    x[ind] <- NA
-  }
-  x
+	cols <- as.character(cols)
+	x <- rep("NULL", length(cols))
+	names(x) <- cols
+	wanted <- list(...)
+	for(i in 1:length(wanted)) {
+		if(is.null(wanted[[i]])) next
+		if(is.function(wanted[[i]])) include <- wtVariables(cols, wanted[[i]])
+		if(is.list(wanted[[i]])) wanted[[i]] <- unlist(wanted[[i]])
+		if(is.character(wanted[[i]])) include <- wanted[[i]]
+		ind <- match(include, cols, nomatch=0)
+		x[ind] <- NA
+	}
+	x
 }
